/*
	(c)Copyright	All rights reserved.
		K.Masuda	2021.12
		and Shirhill family.
			< Color2.c >

		These materials inherit <<CryptOne>>
*/
/*
	(c)Copyright	All rights reserved.
		K.Masuda	2016.01.18
					2019.02.17
	   k.masuda.52431@gmailcom

		and Shirhill family
		and Apple & Microsoft
			  and Bell.
*/
/*
	(c)Copyright	All rights reserved.
               <<CryptOneX>>
            K.Masuda & Shirhill
             Apple & Microsoft
                 Borland
               Oracle & IBM
             Bell & BSD & MIT
                  NORAD.

                 2020.04
                 2022.03
*/
/*
#define	DEBUG
#define	SEQ
*/

#include	"Color.h"
#include	"Area.h"
#include	"Calc.h"

/*
	RandSine、Rand5_1は利用できません。
	Rand、Seed は特注製品です。

	2021.12.29 K.Masuda
	2022.07.23 K.Masuda
*/

/*
	このプログラムの現在のヘッダファイル(Area.h)の設定は１１カ国の４色
	塗り潰し問題解法となっています。Area.hの変更で地球儀の国数までの
	（最大２５５国までの色分けの方策をモンテカルロ法で見つけます。
	乱数生成のアルゴリズムにエラーがあれば解法結果に失敗が出るでしょう。

	2021.12.29 K.Masuda
*/
static	int			iDepth, iCnt1, iCnt2, iCnt3;
#if	0
static	char		pcInv[ ELEVEN ];
#endif
static	char		pcData[ MAXBUFSZ ];	/* 検索国を順番に */
static	char		pcDt[ MAXBUFSZ ];	/* 検索国を順番に */

#if	0
extern	void	rslv( color_t *, int, int, int );
extern	int		MaxColor( color_t *, int, int );
extern	int		Neigh( color_t *, int, int );
extern	void	setAllSt( color_t *, int, int, int, int );
static	void	setInv( int, int );
#endif

/*
	これらの関数は本ソフト公開に際しての特注精品です。
	K.Masuda	2022.07.23
*/
static	void	Seed( int, int, int );
static	int		Rand( int );

/*	2024.02.27
	RndStr でなく連続固定の番号列(SeqStr)とする。
*/
static	char	*SeqStr( int );
/*	K.Masuda	2024.02.27 */
/*	K.Masuda	2022.07.23	*/

int		main(
int		iArgc,
char	*pcArgv[]
){
	int		iRtn, iLp1, iLp2, iLp3, iTmp, eleven;
	struct	timeval		stTime;
	int		iColor, maxclr;
	char	pcFName[ /*IOFNMLEN*/ 128 ];
	FILE	*pstFid;
	char	pcBuf[ 128 ];

	if( iArgc < 6 ){
		fprintf( stderr,
			"モンテカルロ法による４色問題の解法、利用法は以下の通り\n");
		fprintf( stderr,
			"Color2 <色分け国数> <乱数列発生回数> <結果出力> <色数> <種ファイル> [ R | n ]\n" );
		exit( 1 );
	}

	_TRC_INIT( "Color2" );
	_TRC_ENTR( "main" );

	eleven = /* atoi( pcArgv[ 1 ] ) */ ELEVEN;
	iCnt1 = atoi( pcArgv[ 2 ] );
	strcpy( pcFName, pcArgv[ 3 ] );
	maxclr = atoi( pcArgv[ 4 ] );
	_TRC_DMPS( "pcFName", pcFName );

	_TRC_DMPD( "iArgc", iArgc );
	_TRC_DMPS( "pcArgv[1]", pcArgv[ 1 ] );
	_TRC_DMPS( "pcArgv[2]", pcArgv[ 2 ] );
	_TRC_DMPS( "pcArgv[3]", pcArgv[ 3 ] );
	_TRC_DMPS( "pcArgv[4]", pcArgv[ 1 ] );
	_TRC_DMPS( "pcArgv[5]", pcArgv[ 5 ] );
	_TRC_DMPS( "pcArgv[6]", pcArgv[ 6 ] );
#if 1
	if( *pcArgv[ 6 ] == 'R' && iArgc == 7 ){
		pstFid = FOpen( pcArgv[ 5 ], "r" );
		if( pstFid == NULL ){
			fprintf( stderr, "Error : FOpen(%s,r) : %s\n",
							pcArgv[ 5 ], strerror( errno ) );
			exit( 1 );
		}
		if( fgets( pcBuf, 256, pstFid ) == NULL ){
			fprintf( stderr, "Error : %s is NULL file.\n", pcArgv[ 5 ] );
			exit( 1 );
		}
		stTime.tv_sec = ATOL4( pcBuf, 1 );
		stTime.tv_usec = ATOI4( pcBuf, 2 );
		_TRC_DMPD( "R:tv_sec", stTime.tv_sec );
		_TRC_DMPD( "R:tv_usec", stTime.tv_usec );
		FClose( pstFid );
	}
	else{
		iCnt2 = 0;
		pstFid = FOpen( pcArgv[ 5 ], "w" );
		gettimeofday( &stTime, 0 );
		fprintf( pstFid, "%ld,%d,\n", stTime.tv_sec, stTime.tv_usec );	
		_TRC_DMPD( "n:tv_sec", stTime.tv_sec );
		_TRC_DMPD( "n:tv_usec", stTime.tv_usec );
		FClose( pstFid );
	}
#endif
	_TRC_DMPS( "pcFName", pcFName );
	setFid( pstFid = FOpen( pcFName, "w" ) );
	if( pstFid == NULL ){
		fprintf( stderr, "Error : FOpen(%s,w) : %s\n",
						pcArgv[ 3 ], strerror( errno ) );
		exit( 1 );
	}
#ifndef	SINE
#ifdef	RAND5_1
	Seed5_1( 0, stTime.tv_sec, stTime.tv_usec );
#else
	/*
		この場合の設定のみ動作します。以下同様。
		#defineは設定しないで下さい。

		2021.12.29 K.Masuda
	*/
	Seed( 0, stTime.tv_sec, stTime.tv_usec );
#endif
#else
	SeedSine( 0, stTime.tv_sec, stTime.tv_usec );
#endif
	/*
		所定の乱数列発生回数まで繰返す
	*/
	for( iLp1 = 0; iLp1 < iCnt1; iLp1 ++ ){
		/*
			１１カ国の国番号を乱数に基づき発生する(初期のArea.hによる場合)
		*/
		strcpy( pcData,
#ifdef	SEQ
						SeqStr( eleven ) );
#else
						RndStr( eleven, 0,
#ifndef	SINE
#ifdef	RAND5_1
								Rand5_1 ) );
#else
								Rand ) );
#endif
#else
								RandSine ) );
#endif
#endif
		/*
			pcData には色分けの方策国番号が設定されます
		*/
		iTmp = (int)strlen( pcData );
		_NOTRC_DMPD( "iLp1", iLp1 );
		_TRC_DMPS( "pcData", pcData );
		_TRC_DMPD( "strlen(pcData", iTmp );
		if( iTmp != eleven * 4 ){
			fprintf( stderr, "Error : iLp1=%06d,%s\n", iLp1, pcData );
			continue;
		}
		/* 状態変数を初期値に設定 */
		for( iLp2 = 0; iLp2 < eleven; iLp2 ++ ){
			/* 国番号 iLp2 の初期状態を設定 */
			iTmp = ATOI4( pcData, iLp2 + 1 );
			/* 深さiLp2のIDをiTmpとする */
			setStat( iLp2, ID, iTmp );
			_NOTRC_DUMP( "setAllSt() の直前" );
			/* 深さiLp2のその他の状態は"-1,0"初期状態とする */
			setAllSt( iLp2, -1, 0, Area );
		}

	/*
		Area[]には初期値が設定済
	*/

	/*
			４色問題解法の関数を呼出す
	*/
#if	0
		iDepth = -1;
#else
		iDepth = 0;
#endif
		for( iLp2 = 0; iLp2 < maxclr; iLp2 ++ ){
			iColor = iLp2;
			_NOTRC_DMPD( "初期値:iDepth", iDepth );
			_NOTRC_DMPD( "eleven", eleven );
			_NOTRC_DMPD( "初期値:iColor", iColor );
			_NOTRC_DMPD( "iLp1", iLp1 );

			rslv( (color_t *)Area, eleven, iDepth, iColor, maxclr ); 

			_NOTRC_DMPD( "戻り値:iDepth", iDepth );
			_NOTRC_DMPD( "iLp1", iLp1 );
			_NOTRC_DMPD( "戻り値:iColor", iColor );
		}
		_NOTRC_DMPD( "iDepth", iDepth );
	}
	FClose( pstFid );

	_TRC_EXIT();
	_TRC_TERM();
	exit( 0 );
}

static	void	Seed(
int		iId,
int		sec,
int		usec
){
	srand( sec + usec );
}

static	int		Rand(
int		iId
){
	return( rand() );
}

static	char	*SeqStr(
int		eleven
){
	int		iLp1;
	char	pcBuf[ 10 ];

	*pcDt = 0x00;
	for( iLp1 = 0; iLp1 < eleven; iLp1 ++ ){
		sprintf( pcBuf, "%03d,", iLp1 );
		strcat( pcDt, pcBuf );
	}

	return( pcDt );
}
